#include <sourcemod>
#include <vip_core>

#pragma semicolon 1
#pragma newdecls required
#pragma tabsize 0

public Plugin myinfo = 
{
	name = "Multi Vip Test",
	author = "JDW",
	version = "1.4",
	url = "JDW#0930"
};

#define MAX_GROUPS 64

Database _database;

EngineVersion engine;

int times[MAX_GROUPS],
    intervals[MAX_GROUPS],
    counter;

char groups[MAX_GROUPS][64];

Menu hMenu;

public void OnPluginStart()
{
    if(VIP_IsVIPLoaded())
    {
        VIP_OnVIPLoaded();
    }

    RegConsoleCmd("sm_viptest", Command_VipTest);

    RegAdminCmd("sm_mvtstats", Command_MVTStats, ADMFLAG_ROOT);
    RegAdminCmd("sm_mvtreload", Command_MVTReload, ADMFLAG_ROOT);

	if(engine == Engine_SourceSDK2006 || engine == Engine_Left4Dead || engine == Engine_Left4Dead2)
	{
		LoadTranslations("mvt_old.phrases");
	}
	else 
	{
		LoadTranslations("mvt.phrases");
	}
}

public Action Command_VipTest(int client, int args)
{
    if(hMenu)
    {
        hMenu.Display(client, MENU_TIME_FOREVER);
    }
    else 
    {
        PrintToChat(client, "%T%T", "Prefix", client, "Menu not available");
    }
    return Plugin_Handled;
}

public Action Command_MVTStats(int client, int args)
{
    PrintStats(client);
    return Plugin_Handled;
}

public Action Command_MVTReload(int client, int args)
{
    ConfigLoad(client);
    return Plugin_Handled;
}

public void VIP_OnVIPLoaded()
{
    if(_database)
    {
        delete _database;
    }

    char error[256], driver[2], query[512];
    bool mysql;

    if(SQL_CheckConfig("mvt"))
    {
        _database = SQL_Connect("mvt", false, error, 256);
    }
    else 
    {
        _database = SQLite_UseDatabase("mvt", error, 256);
    }

    if(!ConnectionCheck())
    {
        SetFailState("Failed to connect to database, Error: %s", error);
    }

    _database.Driver.GetIdentifier(driver, 2);
    if(driver[0] == 'm')
    {
        mysql = true;
    }

    _database.Format(query, 512, "CREATE TABLE IF NOT EXISTS `mvt_users` (\
                                 `id` INTEGER PRIMARY KEY %s, \
                                 `steam` INTEGER, \
                                 `test_group` varchar(64), \
                                 `next_test` INTEGER)%s",
                                 mysql ? "AUTO_INCREMENT" : "AUTOINCREMENT", mysql ? " CHARSET=utf8 COLLATE utf8_general_ci" : ";");
    _database.Query(SQLQuery_CreateTable, query, _, DBPrio_High);
}

public void SQLQuery_CreateTable(Database database, DBResultSet result, const char[] error, any data)
{
    if(!CheckResult(result, error))
    {
        SetFailState("Failed to create tables");
        return;
    }

    database.SetCharset("utf8");

    DeleteExpiredEntries();
}

public void OnMapStart()
{
    ConfigLoad();
}

void DeleteExpiredEntries()
{
    if(!ConnectionCheck())
    {
        return;
    }

    char query[256];
    _database.Format(query, 256, "DELETE FROM `mvt_users` WHERE `next_test` < '%i' AND `next_test` != 0", GetTime());
    _database.Query(SQLQuery_DeleteExpiredEntries, query);   
}

public void SQLQuery_DeleteExpiredEntries(Database database, DBResultSet result, const char[] error, any data)
{
    CheckResult(result, error);
}

void PrintStats(const int client)
{
    if(!ConnectionCheck())
    {
        return;
    }

    _database.Query(SQLQuery_PrintStats, "SELECT MAX(`id`) FROM `mvt_users`", client);
}

public void SQLQuery_PrintStats(Database database, DBResultSet result, const char[] error, int client)
{
    if(!CheckResult(result, error))
    {
        return;
    }

    if(result.FetchRow() && IsClientInGame(client))
    {
        ESPrintToChat(client, "%T%T", "Prefix", client, "Stats", client, result.FetchInt(0));
    }
}

void CheckForTest(const int client, const int index)
{
    if(!ConnectionCheck())
    {
        return;
    }

    if(VIP_IsClientVIP(client))
    {
        ESPrintToChat(client, "%T%T", "Prefix", client, "Is Vip", client);
        return;
    }

    ArrayList data = new ArrayList();
    data.Push(client);
    data.Push(index);

    char query[256];
    _database.Format(query, 256, "SELECT `next_test` FROM `mvt_users` WHERE `steam` = '%i' AND `test_group` = '%s' ORDER BY ID DESC LIMIT 1",
                          GetSteamAccountID(client), groups[index], GetTime());
    _database.Query(SQLQuery_CheckForTest, query, data);
}

public void SQLQuery_CheckForTest(Database database, DBResultSet result, const char[] error, ArrayList data)
{
    if(!CheckResult(result, error))
    {
        data.Close();
        return;
    }

    int client = data.Get(0);
    int index = data.Get(1);
    data.Close();

    if(IsClientInGame(client))
    {
        if(result.FetchRow())
        {
            int next_test = result.FetchInt(0);

            if(next_test != 0)
            {
                if(next_test < GetTime())
                {
                    GiveGroup(client, index);
                }
                else 
                {
                    char buffer[128];
                    GetStringTime(next_test - GetTime(), buffer, 128);
                    ESPrintToChat(client, "%T%T", "Prefix", client, "Test 2", client, buffer);
                }

                return;
            }
            else 
            {
                ESPrintToChat(client, "%T%T", "Prefix", client, "Test 1", client);
                return;
            }
        }

        GiveGroup(client, index);
    }
}

void GiveGroup(const int client, const int index)
{
    if(!ConnectionCheck())
    {
        return;
    }

    ArrayList data = new ArrayList();
    data.Push(client);
    data.Push(index);

    char query[256];
    _database.Format(query, 256, "INSERT INTO `mvt_users` (`steam`, `test_group`, `next_test`) VALUES ('%i', '%s', '%i')",
                          GetSteamAccountID(client), groups[index], intervals[index] ? intervals[index] + GetTime() : 0);
    _database.Query(SQLQuery_GiveGroup, query, data);
}

public void SQLQuery_GiveGroup(Database database, DBResultSet result, const char[] error, ArrayList data)
{
    if(!CheckResult(result, error))
    {
        data.Close();
        return;
    }

    int client = data.Get(0);
    int index = data.Get(1);
    data.Close();

    if(IsClientInGame(client))
    {
        if(result.InsertId)
        {
            VIP_GiveClientVIP(_, client, times[index], groups[index]);
        }
        else 
        {
            ESPrintToChat(client, "%T%T", "Prefix", client, "Error", client);
        }
    }
}

public int Handler_TestMenu(Menu menu, MenuAction action, int client, int item)
{
    switch(action)
    {
        case MenuAction_Display:
        {
            menu.SetTitle("%T", "Menu", client);
        }
        case MenuAction_Select:
        {
            CheckForTest(client, item);
        }
    }

    return 0;
}

void ConfigLoad(const int client = 0)
{
    static char buffer[PLATFORM_MAX_PATH];
    BuildPath(Path_SM, buffer, PLATFORM_MAX_PATH, "configs/mvt.ini");

    KeyValues kv = new KeyValues("MVT");

    if(!kv.ImportFromFile(buffer) || !kv.GotoFirstSubKey())
    {
        SetFailState("File (%s) not found", buffer);
    }

    kv.Rewind();

    if(hMenu)
    {
        delete hMenu;
    }

    hMenu = new Menu(Handler_TestMenu, MenuAction_Display | MenuAction_Select);

    if(kv.GotoFirstSubKey())
    {
        counter = 0;
        do 
        {
            if(kv.GetSectionName(groups[counter], 64))
            {
                kv.GetString("name", buffer, PLATFORM_MAX_PATH);
                times[counter] = kv.GetNum("Time");
                intervals[counter] = kv.GetNum("Interval");
                hMenu.AddItem("", buffer);
                counter++;
            }
        }   
        while(kv.GotoNextKey(false));
    }

    kv.Close();

    if(client)
    {
        ESPrintToChat(client, "%T%T", "Prefix", client, "Reload", client);
    }
}

bool ConnectionCheck()
{
    if(!_database)
    {
        LogError("Database not available");
        return false;
    }

    return true;
}

bool CheckResult(const DBResultSet r, const char[] error)
{
    if(!r)
    {
        LogError("%s", error);
        return false;
    }

    return true;
}

void GetStringTime(int time, char[] buffer, int maxlength)
{
    static int dims[] = {60, 60, 24, 30, 12, cellmax};
    static char sign[][] = {"с", "м", "ч", "д", "м", "г"};
    static char form[][] = {"%02i%s%s", "%02i%s %s", "%i%s %s"};
    buffer[0] = EOS;
    int i = 0, f = -1;
    bool cond = false;
    while (!cond) {
        if (f++ == 1)
            cond = true;
        do {
            Format(buffer, maxlength, form[f], time % dims[i], sign[i], buffer);
            if (time /= dims[i++], time == 0)
                return;
        } while (cond);
    }
}

static const char colorsBefore[][] = 
{
    "{default}",
    "{team}",
    "{green}",
    "{red}",
    "{lime}",
    "{lightgreen}",
    "{lightred}",
    "{gray}",
    "{lightolive}",
    "{olive}",
    "{lightblue}",
    "{blue}",
    "{purple}",
    "{brightred}"
};

static const char colors[][] = 
{
    "\x01",
    "\x03", 
    "\x04", 
    "\x02", 
    "\x05", 
    "\x06", 
    "\x07", 
    "\x08", 
    "\x09", 
    "\x10", 
    "\x0B", 
    "\x0C", 
    "\x0E", 
    "\x0F"
};

static const int colorsOB[] = 
{
    0xFFFFFF, 
    0x000000, 
    0x00AD00, 
    0xFF0000, 
    0x00FF00, 
    0x99FF99, 
    0xFF4040, 
    0xCCCCCC, 
    0xFFBD6B, 
    0xFA8B00, 
    0x99CCFF, 
    0x3D46FF, 
    0xFA00FA, 
    0xFF6055
};

void ESParse(const int client, char[] message)
{
    if(client && IsClientInGame(client) && !IsFakeClient(client))
    {
        engine = GetEngineVersion();

        if(message[0])
        {
            if(engine != Engine_SourceSDK2006)
            {
                ReplaceString(message, 2048, "{WHITE}", "{DEFAULT}");
            }

            switch(engine)
            {
                case Engine_CSGO:
                {
                    Format(message, 2048, " %s", message);

                    for(int i; i != sizeof(colorsBefore); i++)
                    {
                        ReplaceString(message, 2048, colorsBefore[i], colors[i]);
                    }  
                }
                case Engine_SourceSDK2006, Engine_Left4Dead, Engine_Left4Dead2:
                {
                    for(int i; i != 3; i++)
                    {
                        ReplaceString(message, 2048, colorsBefore[i], colors[i]);
                    }
                }
                case Engine_CSS, Engine_TF2, Engine_DODS, Engine_HL2DM:
                {
                    static char color[16];
                    static int len;

                    len = StrContains(message, colorsBefore[1], false);

                    if(len != -1)
                    {
                        static const int colorsTeam[] = 
                        {
                            0xFFFFFF, 
                            0xCCCCCC, 
                            0xFF4040, 
                            0x99CCFF
                        };

                        FormatEx(color, 16, "\x07%06X", colorsTeam[GetClientTeam(client)]);
                        ReplaceString(message[len], 2048 - len, colorsBefore[1], color);
                    }

                    for(int i; i != sizeof(colorsBefore); i++)
                    {
                        if((len = StrContains(message, colorsBefore[i], false)) != -1)
                        {
                            FormatEx(color, 16, "\x07%06X", colorsOB[i]);
                            ReplaceString(message[len], 2048 - len, colorsBefore[i], color);
                        }
                    }
                }
            }
        }

        SendMessage(client, message);
    }
}

stock void ESPrintToChat(const int client, const char[] format, any ...)
{
    char message[2048];
    VFormat(message, 2048, format, 3);
    ESParse(client, message);
}

stock void ESPrintToChatAll(const char[] format, any ...)
{
    char message[2048];
    VFormat(message, 2048, format, 2);

    for(int i = 1; i <= MaxClients; i++)
    {
        if(IsClientInGame(i) && !IsFakeClient(i))
        {
            ESParse(i, message);
        }
    }
}

void SendMessage(const int client, const char[] text)
{
    Handle hMsg = StartMessageOne("SayText2", client, USERMSG_RELIABLE | USERMSG_BLOCKHOOKS);

    if(hMsg)
    {
        if(GetUserMessageType() == UM_Protobuf)
        {
            Protobuf protoBuf = UserMessageToProtobuf(hMsg);
            protoBuf.SetInt("ent_idx", client);
            protoBuf.SetBool("chat", true);
            protoBuf.SetString("msg_name", text);
            protoBuf.AddString("params", "");
            protoBuf.AddString("params", "");
            protoBuf.AddString("params", "");
            protoBuf.AddString("params", "");
        }
        else 
        {
            BfWrite bfWrite = UserMessageToBfWrite(hMsg);
            bfWrite.WriteByte(client);
            bfWrite.WriteByte(true);
            bfWrite.WriteString(text);
        }
    }

    EndMessage();
}